(*******************************************************************
 *
 *      Component: TRpnButton v1.11
 *         Author: Richard Philip Nguyen (rpnsoft@yahoo.com)
 *            Web: http://rpnsoft.cjb.net
 *    Modified by: Tomasz Trejderowski (ttsoft@koti.com.pl)
 *           Date: 24 November 2000
 *      CopyRight: Richard Philip Nguyen
 *         Status: Freeware
 *
 *    Description: Nice button component that mimics the behaviour of
 *                 buttons found in some homepages
 *
 *          Legal: You are free to use this component in personal and
 *                 commercial applications as long as due credit is given.
 *
 *                 If you change the source code, I ask you to send me
 *                 a copy of your modified code so I can update the
 *                 the component for the next release.
 *
 *        History:
 *                 1.00:  1 Feb 1999 - First release of the component. (rpn)
 *                 1.01:  3 Jul 1999 - Fixed 'Pressed' error.          (ttsoft)
 *                 1.02: 26 Dec 1999 - Added modalresult property      (rpn)
 *                 1.10: 11 Jan 2001 - Modified to descend directly    (rpn)
 *                                     from TButton
 *                 1.11: 23 Jan 2001 - Improved TButton compatibility  (rpn)
 *                                     Draws Shortcut if any
 *
 *******************************************************************)

{$R RpnButton.dcr}

unit RpnButton;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TRpnButton = class(TButton)
  private
    FCanvas: TCanvas;
    IsFocused: Boolean;
    IsMouseOver: Boolean;
    FNormalColor      : TColor;
    FHighlightedColor : TColor;

    procedure CNMeasureItem(var Msg: TWMMeasureItem); message CN_MEASUREITEM;
    procedure CNDrawItem(var Msg: TWMDrawItem); message CN_DRAWITEM;
    procedure CMFontChanged(var Message: TMessage); message CM_FONTCHANGED;
    procedure CMEnabledChanged(var Message: TMessage); message CM_ENABLEDCHANGED;
    procedure WMLButtonDblClk(var Message: TWMLButtonDblClk); message WM_LBUTTONDBLCLK;

    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;

    procedure DrawItem(const DrawItemStruct: TDrawItemStruct);

    procedure SetNormalColor(value: TColor);
    procedure SetHighlightedColor(value: TColor);

  protected
    procedure CreateParams(var Params: TCreateParams); override;

    procedure SetButtonStyle(ADefault: Boolean); override;

  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;

  published
    property NormalColor      : TColor  read FNormalColor      write SetNormalColor;
    property HighlightedColor : TColor  read FHighlightedColor write SetHighlightedColor;
  end;

procedure Register;

implementation

constructor TRpnButton.Create(AOwner: TComponent);
begin
  { Do standard stuff }
  inherited Create(AOwner);

  FCanvas := TCanvas.Create;

  IsMouseOver       := False;
  FHighlightedColor := $004D4D4D;
  FNormalColor      := $00A3A3A3;
  Font.Color        := clWhite;
  Font.Size         := 10;
  Font.Style        := [fsBold];

  { Set width and height of button }
  Width             := 97;
  Height            := 23;
end;

destructor TRpnButton.Destroy;
begin
  FCanvas.Free;
  inherited Destroy;
end;

procedure TRpnButton.CMMouseEnter(var Message: TMessage);
begin
  if (not IsMouseOver) then Invalidate();
end;

procedure TRpnButton.CMMouseLeave(var Message: TMessage);
begin
  if (IsMouseOver) then Invalidate();

end;

procedure TRpnButton.CNMeasureItem(var Msg: TWMMeasureItem);
begin
  with Msg.MeasureItemStruct^ do begin
    itemWidth  := Width;
    itemHeight := Height;
  end;
  Msg.Result := 1;
end;

procedure TRpnButton.CNDrawItem(var Msg: TWMDrawItem);
begin
  DrawItem(Msg.DrawItemStruct^);
  Msg.Result := 1;
end;

procedure TRpnButton.DrawItem(const DrawItemStruct: TDrawItemStruct);
var
  IsDown, IsDefault : Boolean;
  R         : TRect;
  CursorPos : TPoint;
  BtnRect   : TRect;
//  tsSize    : TSize;
//  cx, cy    : longint;
begin
  FCanvas.Handle := DrawItemStruct.hDC;
  try
    R := ClientRect;

    with DrawItemStruct do begin
      IsDown     := (itemState and ODS_SELECTED) <> 0;
      IsDefault  := (itemState and ODS_FOCUS) <> 0;
    end;

    GetCursorPos(CursorPos);
    BtnRect.TopLeft := Parent.ClientToScreen(Point(Left, Top));
    BtnRect.BottomRight := Parent.ClientToScreen(Point(Left + Width,Top + Height));

    IsMouseOver := PtInRect(BtnRect, CursorPos);

    // Draws Outline
    FCanvas.Pen.Width := 3;

    if IsMouseOver then begin
      if IsDown then
        FCanvas.Pen.Color := clBlack
      else
        FCanvas.Pen.Color := clGray
    end
    else
      FCanvas.Pen.Color := clSilver;

    FCanvas.MoveTo(0, R.Bottom-1);
    FCanvas.LineTo(0, 0);
    FCanvas.LineTo(R.Right-1, 0);

    if IsDown then
      FCanvas.Pen.Color := clGray
    else if (IsMouseOver or IsFocused or IsDefault) then
      FCanvas.Pen.Color := clBlack
    else
      FCanvas.Pen.Color := clGray;

{    if (IsFocused or IsDefault) then begin
      if IsDown then
        FCanvas.Pen.Color := clGray
      else
        FCanvas.Pen.Color := clBlack
    end
    else
      FCanvas.Pen.Color := clGray;}

    FCanvas.LineTo(R.Right-1, R.Bottom-1);
    FCanvas.LineTo(0, R.Bottom-1);

    // fills canvas
    if IsMouseOver then
      FCanvas.Brush.Color := FHighlightedColor
    else
      FCanvas.Brush.Color := FNormalColor;

    FCanvas.FillRect(Rect(2, 2, R.Right-2, R.Bottom-2));

    // draws caption
    if Caption<>'' then begin
      FCanvas.Font := Self.Font;
{      GetTextExtentPoint(FCanvas.handle,@Caption[1],Length(Caption),tsSize);
      cx := (R.Right - tsSize.cx) div 2;
      cy := (R.Bottom - tsSize.cy) div 2;
      if IsDown then begin
        Inc(cx);
        Inc(cy);
      end;
      FCanvas.TextOut(cx, cy, Caption);}

      Inc(R.Top,2);
      if IsDown then begin
        Inc(R.Top);
        Inc(R.Left);
      end;
      DrawText(FCanvas.Handle, PChar(Caption), Length(Caption), R,DT_CENTER or DT_VCENTER);
    end;

  finally
    FCanvas.Handle := 0;
  end;
end;

procedure TRpnButton.CMFontChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TRpnButton.CMEnabledChanged(var Message: TMessage);
begin
  inherited;
  Invalidate;
end;

procedure TRpnButton.WMLButtonDblClk(var Message: TWMLButtonDblClk);
begin
  Perform(WM_LBUTTONDOWN, Message.Keys, Longint(Message.Pos));
end;

procedure TRpnButton.SetButtonStyle(ADefault: Boolean);
begin
  if ADefault <> IsFocused then begin
    IsFocused := ADefault;
    Refresh;
  end;
end;

procedure TRpnButton.SetNormalColor(value: TColor);
begin
  FNormalColor := value;
  Invalidate();
end;

procedure TRpnButton.SetHighlightedColor(value: TColor);
begin
  FHighlightedColor := value;
  Invalidate();
end;

procedure TRpnButton.CreateParams(var Params: TCreateParams);
begin
  inherited CreateParams(Params);
  Params.Style := Params.Style OR BS_OWNERDRAW;
end;

procedure Register;
begin
  RegisterComponents('Rpn', [TRpnButton]);
end;

end.
